// uncomment these defines to enable test routines
//#define RIGHT_DIFFICULTY_FREEZE 1
//#define RIGHT_DIFFICULTY_SAFE 1
//#define DIRECTION_DIAGNOSTICS 1

// tried to store pointers are short int, won't compile...
//#define COMPACT_LEVELS

// this only saves them for games that start at Sector 1.
// there's 16 scores to save as there's 4 quadrants and 4 difficulty levels.
#define SAVE_HIGH_SCORES


// sprite usage
//   8 stations (0-7)
//   8 e-type (8-15)
//   1 player ship (16)
//   5 formation (17-21)
//   1 spy ship (22)
//   8 6 i/p-type and 2 explosions (23-30)
// 128 stationary objects, explosions (31-158)
// 159 total

// old
//   20 attacking ships, explosions (23-42)
// 128 stationary objects, explosions (43-170)
// 171 total

#define ETYPE_SPRITE             8
#define PLAYER_SPRITE           16
#define FORMATION_SPRITE        17
#define SPY_SPRITE              22
#define MISC_SPRITES            23
#define FIRST_STATIONARY_SPRITE 31
//#define FIRST_STATIONARY_SPRITE 43


#define FIRST_STATION_MISSILE   4

#define DTP_SPRITE 0
#define DTP_MISSILE 1
#define DTP_DIRECTION 0
#define DTP_TURN 1

#define MAX_SPRITES 159
//#define MAX_SPRITES 171
#define MAX_MISSILES 50
#define MAX_STARS 128

#define NTSC 0
#define PAL 1
#define SECAM 2

#define T1TCR   *(unsigned int*)0xE0008004 // Timer Control Register
#define T1TC    *(unsigned int*)0xE0008008 // Timer Counter

// object color table indexes
#define COLOR_ASTEROID          0   //  0x28  // orange
#define COLOR_MINE              1   //  0x88  // blue
#define COLOR_SHIP              2   //  0x0c  // white, subtract 2 due to lumaboost
#define COLOR_E                 3   //  0x38  // reddish orange
#define COLOR_I                 4   //  0x58  // pink
#define COLOR_P                 5   //  0xb8  // cyan
#define COLOR_SPY               6   //  0x18  // yellow
#define COLOR_STATION           7   //  0xc8  // green
#define COLOR_CONDITION_GREEN   8
#define COLOR_CONDITION_YELLOW  9
#define COLOR_CONDITION_RED     10
#define COLOR_E_LEAD            11
#define COLOR_I_LEAD            12
#define COLOR_P_LEAD            13
#define COLOR_RADAR_ENEMY       14  // 0x48  // red
#define COLOR_EXPLOSIONS        14
#define COLOR_STATION_CORE      46

#define STARFIELD_FRAC_BITS 2

// station direction, used in level layout
#define HS              0
#define VS              1024
#define VS_SHIFT        10

// RADAR_PIXEL is how many screen pixels each dot on the radar represents
// use either 64 or 128
#define RADAR_PIXEL     64
#define WORLD_WIDTH     512
#define WORLD_HEIGHT    1024
#define X_FRACTION_BITS 7
#define Y_FRACTION_BITS 6
#define RADAR_SHIFT     6
#define STATION_X_MASK  511

//#define RADAR_PIXEL     128
//#define WORLD_WIDTH     1024
//#define WORLD_HEIGHT    2048
//#define X_FRACTION_BITS 6
//#define Y_FRACTION_BITS 5
//#define RADAR_SHIFT     7
//#define STATION_X_MASK  1023

// size of safe space around player's starting position
#define PLAYER_BUFFER_SPACE     25

#define STATIONARY_DENSITY      4

// constants for gSpriteState
//#define SS_SPRITE_ACTIVE        0x80
//#define SS_STATION_ONSCREEN     0x40
//#define SS_EXPLOSION_IMAGE      0x40
//#define SS_EXPLOSION_NOT_DEADLY 0x20
//#define SS_STATION_EXPLODING    0x20
//#define SS_EXPLOSION_TIME_MASK  0x0f

// constants for gStation[#].state, 76543210
// 7 = open core shows missile
// 6 = core is open
// 5 = pod 5 active
// ...
// 0 = pod 0 active
#define STS_CORE_MISSILE        0x80
#define STS_CORE_OPEN           0x40
#define STS_POD5_ACTIVE         0x20
#define STS_POD4_ACTIVE         0x10
#define STS_POD3_ACTIVE         0x08
#define STS_POD2_ACTIVE         0x04
#define STS_POD1_ACTIVE         0x02
#define STS_POD0_ACTIVE         0x01
#define STS_ACTIVE_PODS         0x3f

// constants for gStation[#].center, 76543210
// 7 = always open
// 6 = 0 - closing, 1 - opening
// 5 = 0 - no change, 1 = rerender, used in PrepStationGraphics()
//3210 = open amount
#define SCS_ALWAYSOPEN          0x80
#define SCS_OPENING             0x40
#define SCS_RENDER              0x20
#define SCS_AMOUNT              0x0f  // 0-8
#define SCS_FULLY_OPEN             8
#define SCS_FULLY_CLOSED           0

// constants for gSpriteState, bits fedcba9876543210
// f = sprite is onscreen, set by DrawSprite
// e = sprite is deadly if it collides with player's ship
// d = sprite is deadly if it collids with enemy ship
// c = sprite is an explosion (for animation logic)
// b = fast animation speed
// a = use 4x player
// 9 = use 2x player
// 8 = show bonus points when explosion finished OR game over if player sprite
// 7 = lone I-Type or P-Type
// 6 =
// 5 = Game Over
// 43 = frame countdown for explosions
// 210 = explosion time
// 43210 = direction for I-Type, P-Type
#define SPS_SPRITE_ONSCREEN     0x8000
#define SPS_DEADLY_TO_PLAYER    0x4000
#define SPS_DEADLY_TO_ENEMY     0x2000
#define SPS_EXPLOSION           0x1000
#define SPS_FAST_ANIMATION      0x0800
#define SPS_4X_SPRITE           0x0400
#define SPS_2X_SPRITE           0x0200
#define SPS_SHOW_BONUS          0x0100
#define SPS_ENEMYSHIP           0x0080
#define SPS_GAMEOVER            0x0040
#define SPS_FRAME_COUNTDOWN     0x0030
#define SPS_EXPLOSION_TIME_MASK 0x0007 // 0x000f
#define SPS_4_FRAME_EXPLOSION   0x0018 // 0x0030
#define SPS_3_FRAME_EXPLOSION   0x0010 // 0x0020
#define SPS_DECREMENT_FRAME     0x0008 // 0x0010
#define SPS_DIRECTION           0x001f

#define SPS_EXPLOSION_TIME      7       // 15
#define DEADLY_DEBRIS           1
#define SAFE_DEBRIS             0

// For gun cool down use
// *2 = 20
// *3 = 15
// *4 = 10
#define GUN_COOL_DOWN   15
//#define STATION_COOL_DOWN 15

// for joystick processing
#define JOY_RIGHT       0x80
#define JOY_LEFT        0x40
#define JOY_DOWN        0x20
#define JOY_UP          0x10
#define JOY_FIRE        0x80

#define JOY_NO_DIRECTION    ( (SWCHA & 0xF0) == 0xF0)
#define JOY_LEFT_PRESSED    (!(SWCHA & JOY_LEFT)  )
#define JOY_RIGHT_PRESSED   (!(SWCHA & JOY_RIGHT) )
#define JOY_UP_PRESSED      (!(SWCHA & JOY_UP)    )
#define JOY_DOWN_PRESSED    (!(SWCHA & JOY_DOWN)  )
#define JOY_FIRE_PRESSED    (!(INPT4 & JOY_FIRE))
#define JOY_FIRE_RELEASED   (  INPT4 & JOY_FIRE)

// when extra lives are earned - NOTE: these are BCD values
#define FIRST_BONUS 0x20000
#define NEXT_BONUS  0x70000
